@@ -22,15 +22,13 @@
 
 #include "openPMD/IO/AbstractFilePosition.hpp"
 
-
 namespace openPMD
 {
 struct ADIOS1FilePosition : public AbstractFilePosition
 {
-    ADIOS1FilePosition(std::string const& s)
-        : location{s}
-    { }
+    ADIOS1FilePosition(std::string const &s) : location{s}
+    {}
 
     std::string location;
-};  //ADIOS1FilePosition
-} // openPMD
+}; // ADIOS1FilePosition
+} // namespace openPMD
