@@ -20,9 +20,6 @@
  */
 #pragma once
 
-#include "openPMD/config.hpp"
-#include "openPMD/backend/Attributable.hpp"
-#include "openPMD/backend/Container.hpp"
 #include "openPMD/IO/AbstractIOHandler.hpp"
 #include "openPMD/IO/Access.hpp"
 #include "openPMD/IO/Format.hpp"
@@ -38,7 +35,7 @@
 #include "openPMD/version.hpp"
 
 #if openPMD_HAVE_MPI
-#   include <mpi.h>
+#include <mpi.h>
 #endif
 
 #include <map>
@@ -46,10 +43,9 @@
 
 // expose private and protected members for invasive testing
 #ifndef OPENPMD_private
-#   define OPENPMD_private private
+#define OPENPMD_private private
 #endif
 
-
 namespace openPMD
 {
 class ReadIterations;
@@ -58,56 +54,58 @@ class SeriesInterface;
 
 namespace internal
 {
-/**
- * @brief Data members for Series. Pinned at one memory location.
- *
- * (Not movable or copyable)
- *
- */
-class SeriesData : public AttributableData
-{
-public:
-    explicit SeriesData() = default;
-
-    SeriesData( SeriesData const & ) = delete;
-    SeriesData( SeriesData && ) = delete;
-
-    SeriesData & operator=( SeriesData const & ) = delete;
-    SeriesData & operator=( SeriesData && ) = delete;
-
-    virtual ~SeriesData() = default;
-
-    Container< Iteration, uint64_t > iterations{};
-
-    auxiliary::Option< WriteIterations > m_writeIterations;
-    auxiliary::Option< std::string > m_overrideFilebasedFilename;
-    std::string m_name;
-    std::string m_filenamePrefix;
-    std::string m_filenamePostfix;
-    int m_filenamePadding;
-    IterationEncoding m_iterationEncoding{};
-    Format m_format;
     /**
-     *  Whether a step is currently active for this iteration.
-     * Used for group-based iteration layout, see SeriesData.hpp for
-     * iteration-based layout.
-     * Access via stepStatus() method to automatically select the correct
-     * one among both flags.
+     * @brief Data members for Series. Pinned at one memory location.
+     *
+     * (Not movable or copyable)
+     *
      */
-    StepStatus m_stepStatus = StepStatus::NoStep;
-    bool m_parseLazily = false;
-    bool m_lastFlushSuccessful = true;
-}; // SeriesData
+    class SeriesData : public AttributableData
+    {
+    public:
+        explicit SeriesData() = default;
 
-class SeriesInternal;
+        SeriesData(SeriesData const &) = delete;
+        SeriesData(SeriesData &&) = delete;
+
+        SeriesData &operator=(SeriesData const &) = delete;
+        SeriesData &operator=(SeriesData &&) = delete;
+
+        virtual ~SeriesData() = default;
+
+        Container<Iteration, uint64_t> iterations{};
+
+        auxiliary::Option<WriteIterations> m_writeIterations;
+        auxiliary::Option<std::string> m_overrideFilebasedFilename;
+        std::string m_name;
+        std::string m_filenamePrefix;
+        std::string m_filenamePostfix;
+        int m_filenamePadding;
+        IterationEncoding m_iterationEncoding{};
+        Format m_format;
+        /**
+         *  Whether a step is currently active for this iteration.
+         * Used for group-based iteration layout, see SeriesData.hpp for
+         * iteration-based layout.
+         * Access via stepStatus() method to automatically select the correct
+         * one among both flags.
+         */
+        StepStatus m_stepStatus = StepStatus::NoStep;
+        bool m_parseLazily = false;
+        bool m_lastFlushSuccessful = true;
+    }; // SeriesData
+
+    class SeriesInternal;
 } // namespace internal
 
 /** @brief  Implementation for the root level of the openPMD hierarchy.
  *
  * Entry point and common link between all iterations of particle and mesh data.
  *
- * @see https://github.com/openPMD/openPMD-standard/blob/latest/STANDARD.md#hierarchy-of-the-data-file
- * @see https://github.com/openPMD/openPMD-standard/blob/latest/STANDARD.md#iterations-and-time-series
+ * @see
+ * https://github.com/openPMD/openPMD-standard/blob/latest/STANDARD.md#hierarchy-of-the-data-file
+ * @see
+ * https://github.com/openPMD/openPMD-standard/blob/latest/STANDARD.md#iterations-and-time-series
  */
 class SeriesInterface : public AttributableInterface
 {
@@ -121,105 +119,140 @@ class SeriesInterface : public AttributableInterface
 
 protected:
     // Should not be called publicly, only by implementing classes
-    SeriesInterface( internal::SeriesData *, internal::AttributableData * );
+    SeriesInterface(internal::SeriesData *, internal::AttributableData *);
 
 public:
     /**
-     * @return  String representing the current enforced version of the <A HREF="https://github.com/openPMD/openPMD-standard/blob/latest/STANDARD.md#hierarchy-of-the-data-file">openPMD standard</A>.
+     * @return  String representing the current enforced version of the <A
+     * HREF="https://github.com/openPMD/openPMD-standard/blob/latest/STANDARD.md#hierarchy-of-the-data-file">openPMD
+     * standard</A>.
      */
     std::string openPMD() const;
-    /** Set the version of the enforced <A HREF="https://github.com/openPMD/openPMD-standard/blob/latest/STANDARD.md#hierarchy-of-the-data-file">openPMD standard</A>.
+    /** Set the version of the enforced <A
+     * HREF="https://github.com/openPMD/openPMD-standard/blob/latest/STANDARD.md#hierarchy-of-the-data-file">openPMD
+     * standard</A>.
      *
-     * @param   openPMD   String <CODE>MAJOR.MINOR.REVISION</CODE> of the desired version of the openPMD standard.
+     * @param   openPMD   String <CODE>MAJOR.MINOR.REVISION</CODE> of the
+     * desired version of the openPMD standard.
      * @return  Reference to modified series.
      */
-    SeriesInterface& setOpenPMD(std::string const& openPMD);
+    SeriesInterface &setOpenPMD(std::string const &openPMD);
 
     /**
-     * @return  32-bit mask of applied extensions to the <A HREF="https://github.com/openPMD/openPMD-standard/blob/latest/STANDARD.md#hierarchy-of-the-data-file">openPMD standard</A>.
+     * @return  32-bit mask of applied extensions to the <A
+     * HREF="https://github.com/openPMD/openPMD-standard/blob/latest/STANDARD.md#hierarchy-of-the-data-file">openPMD
+     * standard</A>.
      */
     uint32_t openPMDextension() const;
-    /** Set a 32-bit mask of applied extensions to the <A HREF="https://github.com/openPMD/openPMD-standard/blob/latest/STANDARD.md#hierarchy-of-the-data-file">openPMD standard</A>.
+    /** Set a 32-bit mask of applied extensions to the <A
+     * HREF="https://github.com/openPMD/openPMD-standard/blob/latest/STANDARD.md#hierarchy-of-the-data-file">openPMD
+     * standard</A>.
      *
-     * @param   openPMDextension  Unsigned 32-bit integer used as a bit-mask of applied extensions.
+     * @param   openPMDextension  Unsigned 32-bit integer used as a bit-mask of
+     * applied extensions.
      * @return  Reference to modified series.
      */
-    SeriesInterface& setOpenPMDextension(uint32_t openPMDextension);
+    SeriesInterface &setOpenPMDextension(uint32_t openPMDextension);
 
     /**
-     * @return  String representing the common prefix for all data sets and sub-groups of a specific iteration.
+     * @return  String representing the common prefix for all data sets and
+     * sub-groups of a specific iteration.
      */
     std::string basePath() const;
-    /** Set the common prefix for all data sets and sub-groups of a specific iteration.
+    /** Set the common prefix for all data sets and sub-groups of a specific
+     * iteration.
      *
-     * @param   basePath    String of the common prefix for all data sets and sub-groups of a specific iteration.
+     * @param   basePath    String of the common prefix for all data sets and
+     * sub-groups of a specific iteration.
      * @return  Reference to modified series.
      */
-    SeriesInterface& setBasePath(std::string const& basePath);
+    SeriesInterface &setBasePath(std::string const &basePath);
 
     /**
      * @throw   no_such_attribute_error If optional attribute is not present.
-     * @return  String representing the path to mesh records, relative(!) to <CODE>basePath</CODE>.
+     * @return  String representing the path to mesh records, relative(!) to
+     * <CODE>basePath</CODE>.
      */
     std::string meshesPath() const;
-    /** Set the path to <A HREF="https://github.com/openPMD/openPMD-standard/blob/latest/STANDARD.md#mesh-based-records">mesh records</A>, relative(!) to <CODE>basePath</CODE>.
+    /** Set the path to <A
+     * HREF="https://github.com/openPMD/openPMD-standard/blob/latest/STANDARD.md#mesh-based-records">mesh
+     * records</A>, relative(!) to <CODE>basePath</CODE>.
      *
-     * @param   meshesPath  String of the path to <A HREF="https://github.com/openPMD/openPMD-standard/blob/latest/STANDARD.md#mesh-based-records">mesh records</A>, relative(!) to <CODE>basePath</CODE>.
+     * @param   meshesPath  String of the path to <A
+     * HREF="https://github.com/openPMD/openPMD-standard/blob/latest/STANDARD.md#mesh-based-records">mesh
+     * records</A>, relative(!) to <CODE>basePath</CODE>.
      * @return  Reference to modified series.
      */
-    SeriesInterface& setMeshesPath(std::string const& meshesPath);
+    SeriesInterface &setMeshesPath(std::string const &meshesPath);
 
     /**
      * @throw   no_such_attribute_error If optional attribute is not present.
-     * @return  String representing the path to particle species, relative(!) to <CODE>basePath</CODE>.
+     * @return  String representing the path to particle species, relative(!) to
+     * <CODE>basePath</CODE>.
      */
     std::string particlesPath() const;
-    /** Set the path to groups for each <A HREF="https://github.com/openPMD/openPMD-standard/blob/latest/STANDARD.md#particle-records">particle species</A>, relative(!) to <CODE>basePath</CODE>.
+    /** Set the path to groups for each <A
+     * HREF="https://github.com/openPMD/openPMD-standard/blob/latest/STANDARD.md#particle-records">particle
+     * species</A>, relative(!) to <CODE>basePath</CODE>.
      *
-     * @param   particlesPath   String of the path to groups for each <A HREF="https://github.com/openPMD/openPMD-standard/blob/latest/STANDARD.md#particle-records">particle species</A>, relative(!) to <CODE>basePath</CODE>.
+     * @param   particlesPath   String of the path to groups for each <A
+     * HREF="https://github.com/openPMD/openPMD-standard/blob/latest/STANDARD.md#particle-records">particle
+     * species</A>, relative(!) to <CODE>basePath</CODE>.
      * @return  Reference to modified series.
      */
-    SeriesInterface& setParticlesPath(std::string const& particlesPath);
+    SeriesInterface &setParticlesPath(std::string const &particlesPath);
 
     /**
      * @throw   no_such_attribute_error If optional attribute is not present.
-     * @return  String indicating author and contact for the information in the file.
+     * @return  String indicating author and contact for the information in the
+     * file.
      */
     std::string author() const;
     /** Indicate the author and contact for the information in the file.
      *
-     * @param   author  String indicating author and contact for the information in the file.
+     * @param   author  String indicating author and contact for the information
+     * in the file.
      * @return  Reference to modified series.
      */
-    SeriesInterface& setAuthor(std::string const& author);
+    SeriesInterface &setAuthor(std::string const &author);
 
     /**
      * @throw   no_such_attribute_error If optional attribute is not present.
-     * @return  String indicating the software/code/simulation that created the file;
+     * @return  String indicating the software/code/simulation that created the
+     * file;
      */
     std::string software() const;
     /** Indicate the software/code/simulation that created the file.
      *
-     * @param   newName    String indicating the software/code/simulation that created the file.
-     * @param   newVersion String indicating the version of the software/code/simulation that created the file.
+     * @param   newName    String indicating the software/code/simulation that
+     * created the file.
+     * @param   newVersion String indicating the version of the
+     * software/code/simulation that created the file.
      * @return  Reference to modified series.
      */
-    SeriesInterface& setSoftware(std::string const& newName, std::string const& newVersion = std::string("unspecified"));
+    SeriesInterface &setSoftware(
+        std::string const &newName,
+        std::string const &newVersion = std::string("unspecified"));
 
     /**
      * @throw   no_such_attribute_error If optional attribute is not present.
-     * @return  String indicating the version of the software/code/simulation that created the file.
+     * @return  String indicating the version of the software/code/simulation
+     * that created the file.
      */
     std::string softwareVersion() const;
-    /** Indicate the version of the software/code/simulation that created the file.
+    /** Indicate the version of the software/code/simulation that created the
+     * file.
      *
      * @deprecated Set the version with the second argument of setSoftware()
      *
-     * @param   softwareVersion String indicating the version of the software/code/simulation that created the file.
+     * @param   softwareVersion String indicating the version of the
+     * software/code/simulation that created the file.
      * @return  Reference to modified series.
      */
-    [[deprecated("Set the version with the second argument of setSoftware()")]]
-    SeriesInterface& setSoftwareVersion(std::string const& softwareVersion);
+    [[deprecated(
+        "Set the version with the second argument of "
+        "setSoftware()")]] SeriesInterface &
+    setSoftwareVersion(std::string const &softwareVersion);
 
     /**
      * @throw   no_such_attribute_error If optional attribute is not present.
@@ -231,61 +264,79 @@ public:
      * @param   date    String indicating the date of creation.
      * @return  Reference to modified series.
      */
-    SeriesInterface& setDate(std::string const& date);
+    SeriesInterface &setDate(std::string const &date);
 
     /**
      * @throw   no_such_attribute_error If optional attribute is not present.
-     * @return  String indicating dependencies of software that were used to create the file.
+     * @return  String indicating dependencies of software that were used to
+     * create the file.
      */
     std::string softwareDependencies() const;
     /** Indicate dependencies of software that were used to create the file.
      *
-     * @param   newSoftwareDependencies String indicating dependencies of software that were used to create the file (semicolon-separated list if needed).
+     * @param   newSoftwareDependencies String indicating dependencies of
+     * software that were used to create the file (semicolon-separated list if
+     * needed).
      * @return  Reference to modified series.
      */
-    SeriesInterface& setSoftwareDependencies(std::string const& newSoftwareDependencies);
+    SeriesInterface &
+    setSoftwareDependencies(std::string const &newSoftwareDependencies);
 
     /**
      * @throw   no_such_attribute_error If optional attribute is not present.
-     * @return  String indicating the machine or relevant hardware that created the file.
+     * @return  String indicating the machine or relevant hardware that created
+     * the file.
      */
     std::string machine() const;
     /** Indicate the machine or relevant hardware that created the file.
      *
-     * @param   newMachine String indicating the machine or relevant hardware that created the file (semicolon-separated list if needed)..
+     * @param   newMachine String indicating the machine or relevant hardware
+     * that created the file (semicolon-separated list if needed)..
      * @return  Reference to modified series.
      */
-    SeriesInterface& setMachine(std::string const& newMachine);
+    SeriesInterface &setMachine(std::string const &newMachine);
 
     /**
      * @return  Current encoding style for multiple iterations in this series.
      */
     IterationEncoding iterationEncoding() const;
-    /** Set the <A HREF="https://github.com/openPMD/openPMD-standard/blob/latest/STANDARD.md#iterations-and-time-series">encoding style</A> for multiple iterations in this series.
-     * A preview on the <A HREF="https://github.com/openPMD/openPMD-standard/pull/250">openPMD 2.0 variable-based iteration encoding</A> can be activated with this call.
-     * Making full use of the variable-based iteration encoding requires (1) explicit support by the backend (available only in ADIOS2) and (2) use of the openPMD streaming API.
-     * In other backends and without the streaming API, only one iteration/snapshot may be written in the variable-based encoding, making this encoding a good choice for single-snapshot data dumps.
+    /** Set the <A
+     * HREF="https://github.com/openPMD/openPMD-standard/blob/latest/STANDARD.md#iterations-and-time-series">encoding
+     * style</A> for multiple iterations in this series. A preview on the <A
+     * HREF="https://github.com/openPMD/openPMD-standard/pull/250">openPMD 2.0
+     * variable-based iteration encoding</A> can be activated with this call.
+     * Making full use of the variable-based iteration encoding requires (1)
+     * explicit support by the backend (available only in ADIOS2) and (2) use of
+     * the openPMD streaming API. In other backends and without the streaming
+     * API, only one iteration/snapshot may be written in the variable-based
+     * encoding, making this encoding a good choice for single-snapshot data
+     * dumps.
      *
-     * @param   iterationEncoding   Desired <A HREF="https://github.com/openPMD/openPMD-standard/blob/latest/STANDARD.md#iterations-and-time-series">encoding style</A> for multiple iterations in this series.
+     * @param   iterationEncoding   Desired <A
+     * HREF="https://github.com/openPMD/openPMD-standard/blob/latest/STANDARD.md#iterations-and-time-series">encoding
+     * style</A> for multiple iterations in this series.
      * @return  Reference to modified series.
      */
-    SeriesInterface& setIterationEncoding(IterationEncoding iterationEncoding);
+    SeriesInterface &setIterationEncoding(IterationEncoding iterationEncoding);
 
     /**
-     * @return  String describing a <A HREF="https://github.com/openPMD/openPMD-standard/blob/latest/STANDARD.md#iterations-and-time-series">pattern</A> describing how to access single iterations in the raw file.
+     * @return  String describing a <A
+     * HREF="https://github.com/openPMD/openPMD-standard/blob/latest/STANDARD.md#iterations-and-time-series">pattern</A>
+     * describing how to access single iterations in the raw file.
      */
     std::string iterationFormat() const;
-    /** Set a <A HREF="https://github.com/openPMD/openPMD-standard/blob/latest/STANDARD.md#iterations-and-time-series">pattern</A> describing how to access single iterations in the raw file.
+    /** Set a <A
+     * HREF="https://github.com/openPMD/openPMD-standard/blob/latest/STANDARD.md#iterations-and-time-series">pattern</A>
+     * describing how to access single iterations in the raw file.
      *
-     * @param   iterationFormat String with the iteration regex <CODE>\%T</CODE> defining either
-     *                          the series of files (fileBased)
-     *                          or the series of groups within a single file (groupBased)
-     *                          that allows to extract the iteration from it.
-     *                          For fileBased formats the iteration must be included in the file name.
-     *                          The format depends on the selected iterationEncoding method.
+     * @param   iterationFormat String with the iteration regex <CODE>\%T</CODE>
+     * defining either the series of files (fileBased) or the series of groups
+     * within a single file (groupBased) that allows to extract the iteration
+     * from it. For fileBased formats the iteration must be included in the file
+     * name. The format depends on the selected iterationEncoding method.
      * @return  Reference to modified series.
      */
-    SeriesInterface& setIterationFormat(std::string const& iterationFormat);
+    SeriesInterface &setIterationFormat(std::string const &iterationFormat);
 
     /**
      * @return String of a pattern for file names.
@@ -294,10 +345,11 @@ public:
 
     /** Set the pattern for file names.
      *
-     * @param   name    String of the pattern for file names. Must include iteration regex <CODE>\%T</CODE> for fileBased data.
+     * @param   name    String of the pattern for file names. Must include
+     * iteration regex <CODE>\%T</CODE> for fileBased data.
      * @return  Reference to modified series.
      */
-    SeriesInterface& setName(std::string const& name);
+    SeriesInterface &setName(std::string const &name);
 
     /** The currently used backend
      *
@@ -311,60 +363,64 @@ public:
      */
     void flush();
 
-OPENPMD_private:
+    OPENPMD_private:
     static constexpr char const * const BASEPATH = "/data/%T/";
 
     struct ParsedInput;
     using iterations_t = decltype(internal::SeriesData::iterations);
     using iterations_iterator = iterations_t::iterator;
 
-    internal::SeriesData * m_series = nullptr;
+    internal::SeriesData *m_series = nullptr;
 
-    inline internal::SeriesData & get()
+    inline internal::SeriesData &get()
     {
-        if( m_series )
+        if (m_series)
         {
             return *m_series;
         }
         else
         {
             throw std::runtime_error(
-                "[Series] Cannot use default-constructed Series." );
+                "[Series] Cannot use default-constructed Series.");
         }
     }
 
-    inline internal::SeriesData const & get() const
+    inline internal::SeriesData const &get() const
     {
-        if( m_series )
+        if (m_series)
         {
             return *m_series;
         }
         else
         {
             throw std::runtime_error(
-                "[Series] Cannot use default-constructed Series." );
-        }    }
+                "[Series] Cannot use default-constructed Series.");
+        }
+    }
 
-    std::unique_ptr< ParsedInput > parseInput(std::string);
-    void init(std::shared_ptr< AbstractIOHandler >, std::unique_ptr< ParsedInput >);
-    void initDefaults( IterationEncoding );
+    std::unique_ptr<ParsedInput> parseInput(std::string);
+    void init(std::shared_ptr<AbstractIOHandler>, std::unique_ptr<ParsedInput>);
+    void initDefaults(IterationEncoding);
     /**
      * @brief Internal call for flushing a Series.
      *
      * Any flushing of the Series will pass through this call.
-     * 
+     *
      * @param begin Start of the range of iterations to flush.
      * @param end End of the range of iterations to flush.
-     * @param level Flush level, as documented in AbstractIOHandler.hpp.
+     * @param flushParams Flush params, as documented in AbstractIOHandler.hpp.
      * @param flushIOHandler Tasks will always be enqueued to the backend.
      *     If this flag is true, tasks will be flushed to the backend.
      */
-    std::future< void > flush_impl(
+    std::future<void> flush_impl(
         iterations_iterator begin,
         iterations_iterator end,
-        FlushLevel level,
-        bool flushIOHandler = true );
-    void flushFileBased( iterations_iterator begin, iterations_iterator end );
+        internal::FlushParams flushParams,
+        bool flushIOHandler = true);
+    void flushFileBased(
+        iterations_iterator begin,
+        iterations_iterator end,
+        internal::FlushParams flushParams);
     /*
      * Group-based and variable-based iteration layouts share a lot of logic
      * (realistically, the variable-based iteration layout only throws out
@@ -372,19 +428,22 @@ OPENPMD_private:
      * As a convention, methods that deal with both layouts are called
      * .*GorVBased, short for .*GroupOrVariableBased
      */
-    void flushGorVBased( iterations_iterator begin, iterations_iterator end );
+    void flushGorVBased(
+        iterations_iterator begin,
+        iterations_iterator end,
+        internal::FlushParams flushParams);
     void flushMeshesPath();
     void flushParticlesPath();
-    void readFileBased( );
-    void readOneIterationFileBased( std::string const & filePath );
+    void readFileBased();
+    void readOneIterationFileBased(std::string const &filePath);
     /**
      * Note on re-parsing of a Series:
      * If init == false, the parsing process will seek for new
      * Iterations/Records/Record Components etc.
      */
-    void readGorVBased( bool init = true );
+    void readGorVBased(bool init = true);
     void readBase();
-    std::string iterationFilename( uint64_t i );
+    std::string iterationFilename(uint64_t i);
 
     enum class IterationOpened : bool
     {
@@ -397,21 +456,20 @@ OPENPMD_private:
      * Only open if the iteration is dirty and if it is not in deferred
      * parse state.
      */
-    IterationOpened openIterationIfDirty( uint64_t index, Iteration iteration );
+    IterationOpened openIterationIfDirty(uint64_t index, Iteration iteration);
     /*
      * Open an iteration. Ensures that the iteration's m_closed status
      * is set properly and that any files pertaining to the iteration
      * is opened.
      * Does not create files when called in CREATE mode.
      */
-    void openIteration( uint64_t index, Iteration iteration );
+    void openIteration(uint64_t index, Iteration iteration);
 
     /**
      * Find the given iteration in Series::iterations and return an iterator
      * into Series::iterations at that place.
      */
-    iterations_iterator
-    indexOf( Iteration const & );
+    iterations_iterator indexOf(Iteration const &);
 
     /**
      * @brief In step-based IO mode, begin or end an IO step for the given
@@ -428,39 +486,40 @@ OPENPMD_private:
      * @param iteration The actual Iteration object.
      * @return AdvanceStatus
      */
-    AdvanceStatus
-    advance(
+    AdvanceStatus advance(
         AdvanceMode mode,
-        internal::AttributableData & file,
+        internal::AttributableData &file,
         iterations_iterator it,
-        Iteration & iteration );
+        Iteration &iteration);
 }; // SeriesInterface
 
 namespace internal
 {
-class SeriesInternal : public SeriesData, public SeriesInterface
-{
-    friend struct SeriesShared;
-    friend class openPMD::Iteration;
-    friend class openPMD::Series;
-    friend class openPMD::Writable;
+    class SeriesInternal
+        : public SeriesData
+        , public SeriesInterface
+    {
+        friend struct SeriesShared;
+        friend class openPMD::Iteration;
+        friend class openPMD::Series;
+        friend class openPMD::Writable;
 
-public:
+    public:
 #if openPMD_HAVE_MPI
-    SeriesInternal(
-        std::string const & filepath,
-        Access at,
-        MPI_Comm comm,
-        std::string const & options = "{}" );
+        SeriesInternal(
+            std::string const &filepath,
+            Access at,
+            MPI_Comm comm,
+            std::string const &options = "{}");
 #endif
 
-    SeriesInternal(
-        std::string const & filepath,
-        Access at,
-        std::string const & options = "{}" );
-    // @todo make AttributableInterface<>::linkHierarchy non-virtual
-    virtual ~SeriesInternal();
-};
+        SeriesInternal(
+            std::string const &filepath,
+            Access at,
+            std::string const &options = "{}");
+        // @todo make AttributableInterface<>::linkHierarchy non-virtual
+        virtual ~SeriesInternal();
+    };
 } // namespace internal
 
 /** @brief  Root level of the openPMD hierarchy.
@@ -470,26 +529,28 @@ public:
  * An instance can be created either directly via the given constructors or via
  * the SeriesBuilder class.
  *
- * @see https://github.com/openPMD/openPMD-standard/blob/latest/STANDARD.md#hierarchy-of-the-data-file
- * @see https://github.com/openPMD/openPMD-standard/blob/latest/STANDARD.md#iterations-and-time-series
+ * @see
+ * https://github.com/openPMD/openPMD-standard/blob/latest/STANDARD.md#hierarchy-of-the-data-file
+ * @see
+ * https://github.com/openPMD/openPMD-standard/blob/latest/STANDARD.md#iterations-and-time-series
  */
 class Series : public SeriesInterface
 {
 private:
-    std::shared_ptr< internal::SeriesInternal > m_series;
+    std::shared_ptr<internal::SeriesInternal> m_series;
 
     // constructor from private parts
-    Series( std::shared_ptr< internal::SeriesInternal > );
+    Series(std::shared_ptr<internal::SeriesInternal>);
 
 public:
     explicit Series();
 
 #if openPMD_HAVE_MPI
     Series(
-        std::string const & filepath,
+        std::string const &filepath,
         Access at,
         MPI_Comm comm,
-        std::string const & options = "{}" );
+        std::string const &options = "{}");
 #endif
 
     /**
@@ -502,13 +563,13 @@ public:
      *      to a JSON textfile, prepended by an at sign '@'.
      */
     Series(
-        std::string const & filepath,
+        std::string const &filepath,
         Access at,
-        std::string const & options = "{}" );
+        std::string const &options = "{}");
 
     virtual ~Series() = default;
 
-    Container< Iteration, uint64_t > iterations;
+    Container<Iteration, uint64_t> iterations;
 
     /**
      * @brief Is this a usable Series object?
