@@ -20,29 +20,29 @@
  */
 #pragma once
 
-#include "openPMD/backend/PatchRecordComponent.hpp"
 #include "openPMD/backend/BaseRecord.hpp"
+#include "openPMD/backend/PatchRecordComponent.hpp"
 
-#include <unordered_map>
 #include <string>
-
+#include <unordered_map>
 
 namespace openPMD
 {
-class PatchRecord : public BaseRecord< PatchRecordComponent >
+class PatchRecord : public BaseRecord<PatchRecordComponent>
 {
-    friend class Container< PatchRecord >;
+    friend class Container<PatchRecord>;
     friend class ParticleSpecies;
     friend class ParticlePatches;
 
 public:
-    PatchRecord& setUnitDimension(std::map< UnitDimension, double > const&);
+    PatchRecord &setUnitDimension(std::map<UnitDimension, double> const &);
     ~PatchRecord() override = default;
 
 private:
     PatchRecord() = default;
 
-    void flush_impl(std::string const&) override;
+    void
+    flush_impl(std::string const &, internal::FlushParams const &) override;
     void read() override;
-};  //PatchRecord
-} // openPMD
+}; // PatchRecord
+} // namespace openPMD
